package com.dzeio.openhealth.connectors.samsunghealth

import android.app.Activity
import android.content.Intent
import android.os.Handler
import android.os.Looper
import android.util.Log
import com.dzeio.openhealth.MainActivity
import com.dzeio.openhealth.connectors.ConnectorInterface
import com.dzeio.openhealth.data.weight.Weight
import com.samsung.android.sdk.healthdata.*
import com.samsung.android.sdk.healthdata.HealthConstants.StepCount
import com.samsung.android.sdk.healthdata.HealthDataStore.ConnectionListener
import com.samsung.android.sdk.healthdata.HealthPermissionManager.*
import java.util.*


class SamsungHealth(
    private val context: Activity
) : ConnectorInterface {

    companion object {
        const val TAG = "SamsungHealthConnector"
    }

    private val listener = object : ConnectionListener {
        override fun onConnected() {
            Log.d(TAG, "Connected!")
            if (isPermissionAcquired()) {
                reporter.start()
            } else {
                requestPermission()
            }
        }
        override fun onConnectionFailed(p0: HealthConnectionErrorResult?) {
            Log.d(TAG, "Health data service is not available.")
        }

        override fun onDisconnected() {
            Log.d(TAG, "Health data service is disconnected.")
        }

    }

    private val store : HealthDataStore = HealthDataStore(context, listener)

    private fun isPermissionAcquired(): Boolean {
        val permKey = PermissionKey(StepCount.HEALTH_DATA_TYPE, PermissionType.READ)
        val pmsManager = HealthPermissionManager(store)
        try {
            // Check whether the permissions that this application needs are acquired
            val resultMap = pmsManager.isPermissionAcquired(setOf(permKey))
            return !resultMap.containsValue(java.lang.Boolean.FALSE)
        } catch (e: java.lang.Exception) {
            Log.e(TAG, "Permission request fails.", e)
        }
        return false
    }

    private fun requestPermission() {
        val permKey = PermissionKey(StepCount.HEALTH_DATA_TYPE, PermissionType.READ)
        val pmsManager = HealthPermissionManager(store)
        try {
            // Show user permission UI for allowing user to change options
            pmsManager.requestPermissions(setOf(permKey), context)
                .setResultListener { result: PermissionResult ->
                    Log.d(TAG, "Permission callback is received.")
                    val resultMap =
                        result.resultMap
                    if (resultMap.containsValue(java.lang.Boolean.FALSE)) {
                        Log.d(TAG, "No Data???")
                    } else {
                        // Get the current step count and display it
                        reporter.start()
                    }
                }
        } catch (e: Exception) {
            Log.e(TAG, "Permission setting fails.", e)
        }
    }

    private val stepCountObserver = object : StepCountReporter.StepCountObserver {
        override fun onChanged(count: Int) {
            Log.d(TAG, "Step reported : $count")
        }
    }

    private val reporter = StepCountReporter(store, stepCountObserver, Handler(Looper.getMainLooper()))

    /**
     * Connector
     */

    override val sourceID: String = "SamsungHealth"

    override fun onRequestPermissionResult(
        requestCode: Int,
        permission: Array<String>,
        grantResult: IntArray
    ) {}

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {}

    override fun importWeight(callback: (weight: Weight, end: Boolean) -> Unit) {
        store.connectService()
    }
}