package com.dzeio.openhealth.ui.main.home

import android.app.Activity.RESULT_OK
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.commit
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentHomeBinding
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.ui.dialogs.AddWeightDialog
import com.dzeio.openhealth.ui.main.list_weight.ListWeightFragment
import com.github.mikephil.charting.components.AxisBase
import com.github.mikephil.charting.components.Description
import com.github.mikephil.charting.components.XAxis
import com.github.mikephil.charting.components.YAxis
import com.github.mikephil.charting.data.Entry
import com.github.mikephil.charting.data.LineData
import com.github.mikephil.charting.data.LineDataSet
import com.github.mikephil.charting.formatter.ValueFormatter
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collectLatest
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList

@AndroidEntryPoint
class HomeFragment : BaseFragment<HomeViewModel, FragmentHomeBinding>(HomeViewModel::class.java) {

    companion object {
        const val TAG = "HomeFragment"
    }

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentHomeBinding
        get() = FragmentHomeBinding::inflate

    // private lateinit var fit: GoogleFit

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        binding.addWeight.setOnClickListener {
            AddWeightDialog().show(requireActivity().supportFragmentManager, null)
        }

        binding.listWeight.setOnClickListener {
            Log.d("T", "Trying to move")

            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavListWeight())

//            requireActivity().supportFragmentManager.commit {
//                replace(R.id.nav_host_fragment_content_main, ListWeightFragment())
//                addToBackStack(null)
//            }
        }

        // Setup Graph
//        binding.weightGraph.gridLabelRenderer.labelFormatter = DateAsXAxisLabelFormatter(requireContext())
//        binding.weightGraph.gridLabelRenderer.numHorizontalLabels = 3 // only 4 because of the space
//        binding.weightGraph.viewport.isXAxisBoundsManual = true
////        binding.weightGraph.gridLabelRenderer.setHumanRounding(false);
//        binding.weightGraph.addSeries(serie)
    }

//    private val entries = LineGraphSeries<DataPoint>()

    private fun updateGraph(list: List<Weight>) {
        if (list.isNotEmpty()) {
            val first = list[0].timestamp
            val last = list[list.size - 1].timestamp
            val weekFirst = last - 604800000
//            binding.weightGraph.viewport.setMinX(if (first > weekFirst) first else weekFirst)
//            binding.weightGraph.viewport.setMaxX(last);


            val entries = ArrayList<Entry>()
            for (item in list) {
                entries.add(Entry(item.timestamp.toFloat(), item.weight))
            }

            val dataSet = LineDataSet(entries, "Label")
//            binding.weightGraph.xAxis.axisMinimum = if (first > weekFirst) first.toFloat() else weekFirst.toFloat()
//            binding.weightGraph.setExtraOffsets(
//                if (first > weekFirst) first.toFloat() else weekFirst.toFloat(),
//                100f,
//                last.toFloat(),
//                0f
//            )
            binding.weightGraph.isAutoScaleMinMaxEnabled = true
            //binding.weightGraph.setvir
            binding.weightGraph.isDragEnabled = true
            binding.weightGraph.isScaleYEnabled = false
            binding.weightGraph.description = Description().apply { isEnabled = false }
            binding.weightGraph.isScaleXEnabled = true
            binding.weightGraph.setPinchZoom(false)
            binding.weightGraph.xAxis.setLabelCount(5, true)
            binding.weightGraph.xAxis.position = XAxis.XAxisPosition.BOTTOM
            binding.weightGraph.axisRight.isEnabled = false
            binding.weightGraph.axisRight.setDrawGridLines(false)
            binding.weightGraph.xAxis.valueFormatter = object : ValueFormatter() {
                override fun getAxisLabel(value: Float, axis: AxisBase?): String {
                    return SimpleDateFormat("yyyy-MM-dd", Locale.getDefault()).format(Date(value.toLong()))
                    //return super.getAxisLabel(value, axis)
                }
            }
            binding.weightGraph.data = LineData(dataSet)
            binding.weightGraph.invalidate()
        }


//        serie.resetData(list
//            .map { DataPoint(Date(it.timestamp), it.weight.toDouble()) }
//            .toTypedArray()
//        )
    }

    override fun onStart() {
        super.onStart()

        lifecycleScope.launchWhenStarted {
            viewModel.fetchWeights().collectLatest {
                if (it.isEmpty()) {
                    return@collectLatest
                }
                updateGraph(it)
            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        Log.d(TAG, "Activity Result!")
        when (resultCode) {
            RESULT_OK -> {
                // fit.performActionForRequestCode(ActionRequestCode.FIND_DATA_SOURCES)
            }
            else -> {
                Log.e(TAG, "Error: $requestCode, $resultCode")
            }
        }
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>,
                                            grantResults: IntArray) {
        when {
            grantResults.isEmpty() -> {
                // If user interaction was interrupted, the permission request
                // is cancelled and you receive empty arrays.
                Log.i(TAG, "User interaction was cancelled.")
            }

                    grantResults[0] == PackageManager.PERMISSION_GRANTED -> {
                        Log.d(TAG, "Granted")
                // Permission was granted.
//                val fitActionRequestCode = ActionRequestCode.values()[requestCode]
//                fitActionRequestCode.let {
//                    // fit.signIn(ActionRequestCode.FIND_DATA_SOURCES)
//                }
            }
            else -> {
                // Permission denied.

                // In this Activity we've chosen to notify the user that they
                // have rejected a core permission for the app since it makes the Activity useless.
                // We're communicating this message in a Snackbar since this is a sample app, but
                // core permissions would typically be best requested during a welcome-screen flow.

                // Additionally, it is important to remember that a permission might have been
                // rejected without asking the user for permission (device policy or "Never ask
                // again" prompts). Therefore, a user interface affordance is typically implemented
                // when permissions are denied. Otherwise, your app could appear unresponsive to
                // touches or interactions which have required permissions.
                Log.e(TAG, "Error")
//                Snackbar.make(
//                    findViewById(R.id.main_activity_view),
//                    R.string.permission_denied_explanation,
//                    Snackbar.LENGTH_INDEFINITE)
//                    .setAction(R.string.settings) {
//                        // Build intent that displays the App settings screen.
//                        val intent = Intent()
//                        intent.action = Settings.ACTION_APPLICATION_DETAILS_SETTINGS
//                        val uri = Uri.fromParts("package",
//                            BuildConfig.APPLICATION_ID, null)
//                        intent.data = uri
//                        intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
//                        startActivity(intent)
//                    }
//                    .show()
            }
        }
    }
}