package com.dzeio.openhealth.ui.main.imports

import android.app.ProgressDialog
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.RequiresApi
import androidx.lifecycle.lifecycleScope
import com.dzeio.openhealth.connectors.ConnectorInterface
import com.dzeio.openhealth.connectors.GoogleFit
//import com.dzeio.openhealth.connectors.GoogleFit
import com.dzeio.openhealth.connectors.samsunghealth.SamsungHealth
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.databinding.FragmentImportBinding
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class ImportFragment : BaseFragment<ImportViewModel, FragmentImportBinding>(ImportViewModel::class.java) {

    companion object {
        const val TAG = "ImportFragment"
    }

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentImportBinding = FragmentImportBinding::inflate

    private lateinit var progressDialog: ProgressDialog

    private lateinit var fit: ConnectorInterface

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        progressDialog = ProgressDialog(requireContext())

        progressDialog.apply {
            setCancelable(false)
            setTitle("Importing from source...")
        }

        binding.importGoogleFit.setOnClickListener {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                importFromGoogleFit()
            }
        }
        binding.importSamsungHealth.setOnClickListener {
            importFromSamsungHealth()
        }
    }

    private fun importFromGoogleFit() {
        progressDialog.show()
        fit = GoogleFit(requireActivity())

        var imported = 0

        lifecycleScope.launchWhenStarted {
            viewModel.deleteFromSource(fit.sourceID)
        }.invokeOnCompletion {
            //progressDialog.show()
            fit.importWeight { weight, end ->
                Log.d("Importer", "Importing $weight")
                weight.source = fit.sourceID
                progressDialog.setTitle("Importing from source... ${++imported}")
                lifecycleScope.launchWhenStarted {
                    viewModel.importWeight(weight)
                }
                if (end) {
                    Log.d("Importer", "Finished Importing")
                    progressDialog.dismiss()
                    return@importWeight
                }

            }
        }


    }

    fun importFromSamsungHealth() {
        progressDialog.show()
        fit = SamsungHealth(requireActivity())

        var imported = 0

        lifecycleScope.launchWhenStarted {
            viewModel.deleteFromSource(fit.sourceID)
        }.invokeOnCompletion {
            //progressDialog.show()
            fit.importWeight { weight, end ->
                Log.d("Importer", "Importing $weight")
                weight.source = fit.sourceID
                progressDialog.setTitle("Importing from source... ${++imported}")
                lifecycleScope.launchWhenStarted {
                    viewModel.importWeight(weight)
                }
                if (end) {
                    Log.d("Importer", "Finished Importing")
                    progressDialog.dismiss()
                    return@importWeight
                }

            }
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        fit.onActivityResult(requestCode, resultCode, data)
    }

    @RequiresApi(Build.VERSION_CODES.O)
    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<String>,
                                            grantResults: IntArray) {
        when {
            grantResults.isEmpty() -> {
                // If user interaction was interrupted, the permission request
                // is cancelled and you receive empty arrays.
                Log.i(TAG, "User interaction was cancelled.")
            }

            grantResults[0] == PackageManager.PERMISSION_GRANTED -> {
                Log.d(TAG, "Granted")
                fit.onRequestPermissionResult(requestCode, permissions, grantResults)
            }
            else -> {
                // Permission denied.

                // In this Activity we've chosen to notify the user that they
                // have rejected a core permission for the app since it makes the Activity useless.
                // We're communicating this message in a Snackbar since this is a sample app, but
                // core permissions would typically be best requested during a welcome-screen flow.

                // Additionally, it is important to remember that a permission might have been
                // rejected without asking the user for permission (device policy or "Never ask
                // again" prompts). Therefore, a user interface affordance is typically implemented
                // when permissions are denied. Otherwise, your app could appear unresponsive to
                // touches or interactions which have required permissions.
                Log.e(TAG, "Error")
//                Snackbar.make(
//                    findViewById(R.id.main_activity_view),
//                    R.string.permission_denied_explanation,
//                    Snackbar.LENGTH_INDEFINITE)
//                    .setAction(R.string.settings) {
//                        // Build intent that displays the App settings screen.
//                        val intent = Intent()
//                        intent.action = Settings.ACTION_APPLICATION_DETAILS_SETTINGS
//                        val uri = Uri.fromParts("package",
//                            BuildConfig.APPLICATION_ID, null)
//                        intent.data = uri
//                        intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
//                        startActivity(intent)
//                    }
//                    .show()
            }
        }
    }
}